@@ -3,7 +3,7 @@ import sys
 from pathlib import Path
 
 from snowfakery.data_gen_exceptions import DataGenError
-
+from snowfakery.utils.versions import check_latest_version
 
 import click
 from snowfakery import version
@@ -23,7 +23,7 @@ class FormatChoice(click.Choice):
 
 def eval_arg(arg):
     if arg.isnumeric():
-        return int(float(arg))
+        return int(arg)
     else:
         return arg
 
@@ -51,6 +51,35 @@ def int_string_tuple(ctx, param, value=None):
     return string, number
 
 
+class VersionMessage:
+    "Class that formats itself as a version message when % interpolated"
+
+    def quick_test(self) -> str:
+        from io import StringIO
+
+        yaml = """
+        - object: Status
+          fields:
+            quote: Shiny and new
+        """
+        out = StringIO()
+        generate_data(StringIO(yaml), output_files=[out], output_format="txt")
+        return "Properly installed" if out.getvalue() else "Unknown installation error"
+
+    def __mod__(self, vals) -> str:
+        return "\n".join(
+            (
+                f"snowfakery version {version}",
+                check_latest_version(version).message,
+                "",
+                f"Program: {__file__}",
+                f"Python: {sys.version}",
+                f"Executable: {sys.executable}",
+                f"Installation: {self.quick_test()}",
+            )
+        )
+
+
 @click.command()
 @click.argument("yaml_file", type=click.File("r"))
 @click.option(
@@ -126,7 +155,7 @@ def int_string_tuple(ctx, param, value=None):
     help="Declarations to mix into the generated mapping file",
     multiple=True,
 )
-@click.version_option(version=version, prog_name="snowfakery")
+@click.version_option(version=version, prog_name="snowfakery", message=VersionMessage())
 def generate_cli(
     yaml_file,
     option=(),
