@@ -5,6 +5,7 @@ from importlib import import_module
 from datetime import date, datetime
 from pathlib import Path
 from unittest.mock import patch
+from functools import wraps
 
 import yaml
 from yaml.representer import Representer
@@ -19,6 +20,7 @@ from numbers import Number
 
 Scalar = Union[str, Number, date, datetime, None]
 FieldDefinition = "snowfakery.data_generator_runtime_object_model.FieldDefinition"
+ObjectRow = "snowfakery.object_rows.ObjectRow"
 
 
 class LineTracker(NamedTuple):
@@ -96,14 +98,23 @@ class PluginContext:
     def field_vars(self):
         return self.interpreter.current_context.field_vars()
 
+    ## TODO: Deprecate this in favour of get_contextual_state
+    ##       which has more smarts about name=, parent= etc.
     def context_vars(self):
-        return self.interpreter.current_context.context_vars(
-            self.plugin.__class__.__name__
+        return self.interpreter.current_context.context_vars(id(self.plugin))
+
+    def local_vars(self):
+        return self.interpreter.current_context.local_vars.setdefault(
+            id(self.plugin), {}
         )
 
+    @property
     def unique_context_identifier(self) -> str:
-        "An identifier that will be unique across iterations (but not portion invocations)"
-        return self.interpreter.current_context.unique_context_identifier
+        """An identifier representing a template context that will be
+        unique across iterations (but not portion invocations). It
+        allows templates that do counting or iteration for a particular
+        template context."""
+        return str(self.interpreter.current_context.unique_context_identifier)
 
     def evaluate_raw(self, field_definition):
         """Evaluate the contents of a field definition"""
@@ -130,6 +141,36 @@ def lazy(func: Any) -> Callable:
     return func
 
 
+def memorable(func: Any) -> Callable:
+    """A memorable function is one that is evaluated once per iteration and its result saved"""
+    func.memorable = True
+
+    @wraps(func)
+    def newfunc(self, *args, **kwargs):
+        return evaluate_memorable_function(self.context, func, self, args, kwargs)
+
+    return newfunc
+
+
+def evaluate_memorable_function(context, func, self, args, kwargs):
+    user_key = kwargs.get("name") or (
+        context.unique_context_identifier,
+        tuple(args),
+        tuple(kwargs.items()),
+    )
+    key = (
+        func.__module__,
+        func.__name__,
+        user_key,
+    )
+    return context.interpreter.get_contextual_state(
+        name=key,
+        parent=kwargs.get("parent", None),
+        reset_every_iteration=False,
+        make_state_func=lambda: func(self, *args, **kwargs),
+    )
+
+
 def resolve_plugins(
     plugin_specs: List[Tuple[str, object]], search_paths: List[Union[str, Path]]
 ):
@@ -239,6 +280,28 @@ class PluginResult:
     def __str__(self):
         return str(self.result)
 
+    @classmethod
+    def _from_continuation(cls, args):
+        return cls(**args)
+
+    def __init_subclass__(cls, **kwargs):
+        super().__init_subclass__(**kwargs)
+        _register_for_continuation(cls)
+
+
+def _register_for_continuation(cls):
+    SnowfakeryDumper.add_representer(cls, Representer.represent_object)
+    yaml.SafeLoader.add_constructor(
+        f"tag:yaml.org,2002:python/object/apply:{cls.__module__}.{cls.__name__}",
+        lambda loader, node: cls._from_continuation(
+            loader.construct_mapping(node.value[0])
+        ),
+    )
+
+
+class PluginResultIterator(PluginResult):
+    pass
+
 
 class PluginOption:
     def __init__(self, name, typ):
@@ -248,16 +311,12 @@ class PluginOption:
     def convert(self, value):
         try:
             return self.type(value)
-        except TypeError as e:
-            raise TypeError(
+        except (TypeError, ValueError) as e:
+            raise exc.DataGenTypeError(
                 f"{self.name} option is wrong type {type(value)} rather than {self.type}",
                 *e.args,
             )
 
 
 # round-trip PluginResult objects through continuation YAML if needed.
-SnowfakeryDumper.add_representer(PluginResult, Representer.represent_object)
-yaml.SafeLoader.add_constructor(
-    "tag:yaml.org,2002:python/object/apply:snowfakery.plugins.PluginResult",
-    lambda loader, node: PluginResult(loader.construct_sequence(node)[0]),
-)
+_register_for_continuation(PluginResult)
