@@ -2,10 +2,11 @@ import random
 from functools import lru_cache
 from datetime import date, datetime
 import dateutil.parser
+import warnings
 from dateutil.relativedelta import relativedelta
 from ast import literal_eval
 
-from typing import Union, List, Tuple
+from typing import Union, List, Tuple, Any
 
 from faker import Faker
 from faker.providers.date_time import Provider as DateProvider
@@ -14,11 +15,11 @@ from .data_gen_exceptions import DataGenError
 
 import snowfakery.data_generator_runtime  # noqa
 from snowfakery.plugins import SnowfakeryPlugin, PluginContext, lazy
+from snowfakery.object_rows import ObjectReference
+from snowfakery.utils.template_utils import StringGenerator
+from snowfakery.standard_plugins.UniqueId import UniqueId
 
 FieldDefinition = "snowfakery.data_generator_runtime_object_model.FieldDefinition"
-ObjectRow = "snowfakery.data_generator_runtime.ObjectRow"
-
-fake = Faker()
 
 # It might make more sense to use context vars for context handling when
 # Python 3.6 is out of the support matrix.
@@ -67,6 +68,16 @@ def render_boolean(context: PluginContext, value: FieldDefinition) -> bool:
 class StandardFuncs(SnowfakeryPlugin):
     class Functions:
         int = int
+        # use ONLY for random_dates
+        # anything else should use the Faker from the Interpreter
+        # which is locale-scoped.
+        _faker_for_dates = Faker(use_weighting=False)
+        _uidgen = None
+
+        def __init__(self, *args, **kwargs):
+            self.snowfakery_filename = StringGenerator(self._snowfakery_filename)
+            self.unique_id = StringGenerator(self._unique_id)
+            self.unique_alpha_code = StringGenerator(self._unique_alpha_code)
 
         def date(
             self,
@@ -111,14 +122,15 @@ class StandardFuncs(SnowfakeryPlugin):
             end_date = try_parse_date(end_date)
 
             try:
-                return fake.date_between(start_date, end_date)
+                return self._faker_for_dates.date_between(start_date, end_date)
             except ValueError as e:
                 if "empty range" not in str(e):
                     raise
             # swallow empty range errors per Python conventions
 
         def i18n_fake(self, locale: str, fake: str):
-            faker = Faker(locale)
+            # deprecated by still here for backwards compatibility
+            faker = Faker(locale, use_weighting=False)
             func = getattr(faker, fake)
             return func()
 
@@ -126,22 +138,34 @@ class StandardFuncs(SnowfakeryPlugin):
             """Pick a random number between min and max like Python's randint."""
             return random.randrange(min, max + 1, step)
 
-        def reference(self, x: Union[ObjectRow, str]):
+        def reference(self, x: Any):
             """YAML-embeddable function to Reference another object."""
             if hasattr(x, "id"):  # reference to an object with an id
                 target = x
             elif isinstance(x, str):  # name of an object
-                obj = self.context.field_vars().get(x)
-                if not obj:
+                # allows dotted paths
+                parts = x.split(".")
+                target = self.context.field_vars().get(parts.pop(0))
+
+                for part in parts:
+                    try:
+                        target = getattr(target, part)
+                    except AttributeError:
+                        raise DataGenError(
+                            f"Expression cannot be evaluated `{x}``. Problem before `{part}`:\n {target}",
+                            None,
+                            None,
+                        )
+
+                if not target:
                     raise DataGenError(f"Cannot find an object named {x}", None, None)
-                if not getattr(obj, "id", None):
+                if not getattr(target, "id", None):
                     raise DataGenError(
-                        f"Reference to incorrect object type {obj}", None, None
+                        f"Reference to incorrect object type {target}", None, None
                     )
-                target = obj
             else:
                 raise DataGenError(
-                    f"Can't get reference to object of type {type(x)}: {x}", None, None
+                    f"Can't get reference to object of type {type(x)}: {x}"
                 )
 
             return target
@@ -215,6 +239,46 @@ class StandardFuncs(SnowfakeryPlugin):
                 probability = parse_weight_str(self.context, probability)
             return probability or when, pick
 
+        def random_reference(self, tablename: str, scope: str = "current-iteration"):
+            """Select a random, already-created row from 'sobject'
+
+            - object: Owner
+              count: 10
+              fields:
+                name: fake.name
+            - object: Pet
+              count: 10
+              fields:
+                ownedBy:
+                    random_reference:
+                        Owner
+
+            See the docs for more info.
+            """
+
+            globls = self.context.interpreter.globals
+            last_id = globls.transients.last_id_for_table(tablename)
+            if last_id:
+                if scope == "prior-and-current-iterations":
+                    first_id = 1
+                    warnings.warn("Global scope is an experimental feature.")
+                elif scope == "current-iteration":
+                    first_id = globls.first_new_id(tablename)
+                    last_id = max(first_id, last_id)
+                else:
+                    raise DataGenError(
+                        f"Scope must be 'prior-and-current-iterations' or 'current-iteration' not {scope}",
+                        None,
+                        None,
+                    )
+                return ObjectReference(tablename, random.randint(first_id, last_id))
+            elif tablename in globls.transients.nicknamed_objects:
+                raise DataGenError(
+                    "Nicknames cannot be used in random_reference", None, None
+                )
+            else:
+                raise DataGenError(f"There is no table named {tablename}", None, None)
+
         @lazy
         def if_(self, *choices: FieldDefinition):
             """Template helper for conditional choices.
@@ -252,5 +316,25 @@ class StandardFuncs(SnowfakeryPlugin):
                 rc = self.context.evaluate_raw(rc)
             return rc
 
+        def _snowfakery_filename(self):
+            template = self.context.field_vars()["template"]
+            return template.filename
+
+        @property
+        def _unique_id_generator(self):
+            if not self._uidgen:
+                self._uidgen = UniqueId(self.context.interpreter).custom_functions()
+
+            return self._uidgen
+
+        def _unique_id(self):
+            return self._unique_id_generator.default_uniqifier.unique_id
+
+        def _unique_alpha_code(self):
+            return self._unique_id_generator.default_alpha_code_generator.unique_id
+
     setattr(Functions, "if", Functions.if_)
     setattr(Functions, "relativedelta", relativedelta)
+    setattr(Functions, "NULL", None)
+    setattr(Functions, "null", None)
+    setattr(Functions, "Null", None)
